local bar = _G[...]
local pieces = {}
local layers = {
	"BACKGROUND",
	"LOW",
	"MEDIUM",
	"HIGH",
	"DIALOG",
	"FULLSCREEN",
	"FULLSCREEN_DIALOG",
	"TOOLTIP",
}
local function Hook(frame)
	if not frame.hooked then
		frame.hooked = true
	else
		return
	end
	local def = frame.defaults
	if def then
		def.frameLayer = 1
		def.frameStrata = 2
		frame.sets.frameLayer = frame.sets.frameLayer or def.frameLayer
		frame.sets.frameStrata = frame.sets.frameStrata or def.frameStrata

	end

	hooksecurefunc(frame, "Layout", function(self)
		if not self.sets then
			return
		end
		local strat = ceil(self.sets.frameStrata)
		local lay = layers[strat]
		self:SetFrameStrata(lay)
		self:SetFrameLevel(ceil(self.sets.frameLayer)+1)
		if self.drop then
			self.drop:SetFrameLevel(ceil(self.sets.frameLayer))
			self.drop:SetFrameStrata(lay)
		end
	end)
end


function bar:Load()
	self.components = self.components or {}
	if #self.components == 0 then
		for i, info in pairs(pieces) do
			local component, key = info.func:New(self), info.id
            if component then
			component.key = key
			if component.defaults then
				self.sets[key] = self.sets[key] or {}
				self.check(component.defaults, self.sets[key])
				component.sets = self.sets[key]
				self[key] = component
			end
			tinsert(self.components, component)
			end
		end
	end
	for id, component in pairs(self.components) do
		if component.Load then
			component:Load()
		end
	end
end

function bar:NewComponent(name, frameType)
	local component = bar.master:CreateClass(frameType)
 	tinsert(pieces, {id = name, func = component})

 	return component
end

function bar:UpdateComponents()
	if self.components then
		for id, component in pairs(self.components) do
			if component.Layout then
				Hook(component)
				component:Layout()
			end
		end
	end
end

--curved fontstring functions
bar.ArcStrings = CreateFrame("Frame")
local ArcStrings = bar.ArcStrings
local component = bar.ArcStrings

ArcStrings.letters =  ArcStrings.letters or {}
ArcStrings.unusedLetters =  ArcStrings.unusedLetters or {}

local allLetters = ArcStrings.letters
local unusedLetters = ArcStrings.unusedLetters

function component:EnableArcStrings(self, ...)
    self.stringManager = self.stringManager or self:CreateTexture(nil, "ARTWORK")
    self.stringManager:SetTexture(0,1,1,.5)
    self.stringManager:ClearAllPoints()
    self.stringManager:SetPoint(self.sets.anchorText, self, self.sets.anchorText, self.sets.textX, self.sets.textY)
    self.stringManager:SetSize(self:GetWidth(), 24)
    self.textWidth = 0
    self.lines = self.lines or {}
    component:SetText(self, " ")
    component:ClearString(self)
    return self.stringManager
end

function ArcStrings:NewLetter()  
    local letter = ArcStrings:CreateFontString(nil, 'ARTWORK', 'TextStatusBarText')
    letter.manager = CreateFrame("Frame")
    letter:SetPoint("Center", letter.manager)
    letter.manager:SetHeight(12)
    letter:SetTextColor(1, 1, 1, 1)
	letter:SetDrawLayer("ARTWORK")
    tinsert(allLetters, letter)
    return letter
end

function component:ReuseLetter(parent)  
    local letter 
    if unusedLetters[1] then
        letter = unusedLetters[1]
        letter:ClearAllPoints()
        letter:SetPoint("Center", letter.manager)
        tremove(unusedLetters, 1)
    end
    return letter
end

function component:GetLetter(self)
    local letter = ArcStrings:ReuseLetter() or ArcStrings:NewLetter()
    letter:SetParent(self)
    return letter
end

function component:FreeLetter(self, letter)
    if not letter then
        return
    end
    letter:SetText("")
    letter:ClearAllPoints()
    letter:SetPoint("Center", letter.manager)
    letter.manager:ClearAllPoints()
    tinsert(unusedLetters, letter)
end

function component:ClearString(self)
     for i, letter in pairs(self.lines) do
        component:FreeLetter(self, letter)
    end
    self.prevText = nil
    wipe(self.lines)   
end

function component:RelocateStringManager(self)
    if not self.stringManager then
        component:EnableArcStrings(self)
    end
    if (self.prevTextY ~= self.sets.textY)
    or (self.prevTextX ~= self.sets.textY)
    or (self.prevanchorText ~= self.sets.anchorText) then
        self.prevTextY = self.sets.textY
        self.prevTextX = self.sets.textX
        self.prevanchorText = self.sets.anchorText
        self.stringManager:ClearAllPoints()
        self.stringManager:SetPoint(self.sets.anchorText, self, self.sets.anchorText, self.sets.textX, self.sets.textY)
        self.stringManager:SetSize(10, 24)
        return true
    end
end

function component:NoChangeMade(self, text)
    if (text ~= self.prevText)
    or (self.prevmaxTextWidth ~= self.sets.maxTextWidth)
    or (self.prevarc ~= self.sets.arc)
    or (component:RelocateStringManager(self))
    or (self.prevCurveOffset ~= self.sets.CurveOffset)
    or (self.prevTextX ~= self.sets.textX)
    or (self.prevTextY ~= self.sets.textY)
    then  
        self.prevTextX = self.sets.textX
        self.prevTextY = self.sets.textY
        self.prevText = text
        self.prevmaxTextWidth = self.sets.maxTextWidth
        self.prevarc = self.sets.arc
        self.prevCurveOffset = self.sets.CurveOffset
    else
        self.prevText = text
        return true
    end
end

function component:SetupLetter(self)



end

function component:GetString(self, text)
    local numLetters = string.len(text)
    local lettterCount = numLetters
    local strip = text
    local anchor = self.sets.anchorText
    local anchRight = string.find(anchor, "Right")
    local lines = self.lines
    local maxWidth = self.sets.maxTextWidth
    local multi = self.sets.multi
    local currentline = 1
    local y = 0
    for i = 1, numLetters do
        local letter = component:GetLetter(self)
        letter:SetParent(self)
        letter:SetDrawLayer("ARTWORK", 8)
        --[[if anchRight then
            letter:SetText(string.sub(strip, lettterCount, lettterCount))
            strip = string.sub(strip, 1, lettterCount-1)
        else--]]
            letter:SetText(string.sub(strip, 1, 1))
            strip = string.sub(strip, 2, lettterCount)
       -- end
        lettterCount = lettterCount - 1
		local stringWidth = letter:GetStringWidth()
		if letter:GetText() == " " then
			stringWidth = stringWidth/2
		end
        local letterWidth = stringWidth/2
        if i < ( maxWidth+1) then
            tinsert(self.lines, letter)
            self.textWidth = (self.textWidth + (letterWidth * self:GetEffectiveScale()))
            letter.manager:SetWidth(letterWidth) 
            letter.manager:ClearAllPoints()
            letter.manager.width = letterWidth
            local pointA, pointB = "Left", "Right"
            --[[if anchRight then 
                pointA, pointB = "Right", "Left"
            end--]]
            if i == 1 then
                letter.manager:SetPoint(pointA, self.stringManager, pointA, 0, 0)
            elseif letter.manager ~= lines[i-1].manager then
                letter.manager:SetPoint("Bottom"..pointA, lines[i-1].manager, "Bottom"..pointB)
            end
        else
            component:FreeLetter(self, letter)
            break
        end
    end
end

function component:ResizeStringManager(self, lines, anchor)
    local firstLetter, lastLetter = lines[1].manager, lines[#lines].manager
    local p
    local width = 0
    if not (firstLetter:GetCenter() and lastLetter:GetCenter()) then
        return
    end
    if anchRight then
        width = (firstLetter:GetCenter() + (firstLetter.width/2))- (lastLetter:GetCenter() - (lastLetter.width/2))
    else
        width = (firstLetter:GetCenter() - (firstLetter.width/2)) - (lastLetter:GetCenter() + (lastLetter.width/2))
    end
    width = math.abs(width)/ self:GetEffectiveScale()
    self.stringManager:SetSize(width, 24)
end


function component:SetText(self, text)
    component:RelocateStringManager(self)
    if component:NoChangeMade(self, text) then
        return
    end
    local lines = self.lines
    local numLetters = string.len(text)
    if numLetters < 1 then
        return
    end
    component:ClearString(self)
    local anchor = self.sets.anchorText
    local anchRight = string.find(anchor, "Right")
    component:RelocateStringManager(self)
    component:GetString(self, text)
    local firstLetter, lastLetter = lines[1].manager, lines[#lines].manager
    local p
    local width = 0
    local firstCenter, lastCenter = firstLetter:GetCenter(), lastLetter:GetCenter()
    local firstWidth, lastWidth = firstLetter.width, lastLetter.width
    if not (firstCenter and lastCenter) then
        return
    end
    if anchRight then
        width = (firstCenter + (firstWidth/2))- (lastCenter - (lastWidth/2))
    else
        width = (firstCenter - (firstWidth/2)) - (lastCenter + (lastWidth/2))
    end
    width = math.abs(width)/ self:GetEffectiveScale()
    self.stringManager:SetSize(width, 24)     
    if (#self.lines > 0) then
        local stringManager = self.stringManager
        local CurveOffset = self.sets.CurveOffset or 0
        CurveOffset = ((CurveOffset/100) * (self:GetWidth()))
		if not (stringManager:GetCenter() or CurveOffset) then
			return
		end
        local Sx = (stringManager:GetCenter() - CurveOffset)--
        local width, height = self:GetSize()
        local a = (height/2)/((width/2)^2)
        local arc = self.sets.arc/100
        for i, letter in pairs(self.lines) do
            local Lx = letter:GetCenter()
            if Lx then
                local lx = math.abs(Sx - Lx)
                local y = (a*(lx^2)) * arc
                letter:ClearAllPoints()
                letter:SetPoint("Center", letter.manager, "Center", 0, y)
            end
        end
    end
end
