local bar = _G[...]
local modName = ...
local title = "AltPower"
--Alt Power Bar
local component = bar:GetTemplate("StatusBar", title)

function component:New(parent)
	local comp = self:Bind(self:CreateNew(parent, title))
		if ( not self.powerName ) then
		self.powerName = ADDITIONAL_POWER_BAR_NAME;
		self.powerIndex = ADDITIONAL_POWER_BAR_INDEX;
	end
	
	return comp
end


local basePower = {Druid = 0, Warlock = 0, Shaman = 0, Mage = 0, Monk = 0, Paladin = 0, Priest = 0, Warrior = 1, Hunter = 3 , ["Death Knight"] = 6, Rogue = 3}

function component:ShouldDisplayPower()
	local unit = self.owner.id
	if UnitHasVehiclePlayerFrameUI(unit) then
		return false;
	end

	if UnitPowerMax(unit, self.powerIndex) == 0 then
		return false;
	end

	local _, class = UnitClass(unit);
	if ALT_MANA_BAR_PAIR_DISPLAY_INFO[class] then
		local powerType = UnitPowerType(unit);
		return ALT_MANA_BAR_PAIR_DISPLAY_INFO[class][powerType];
	end

	return false;
end

function component:OnUpdate()
	if not self:ShouldDisplayPower() then
		self:Hide()
		return
	else
		self:Show()
	end
	local unit = self.owner.id
	local basePower_index = basePower[UnitClass(unit)]
	local dead = UnitIsGhost(unit) or UnitIsDead(unit)
	
	self:SetValues(UnitPower(unit, basePower_index), 0, UnitPowerMax(unit, basePower_index), dead)
	
	local r, g, b
	if ( not UnitIsConnected(unit)) then
		r, g, b = 0.5, 0.5, 0.5
	else
		local powerType, powerToken, altR, altG, altB = UnitPowerType(unit, basePower_index)
		powerType = basePower_index
		local info = PowerBarColor[powerToken]
		if ( info ) then
			r, g, b = info.r, info.g, info.b
		else
			if ( not altR) then
				info = PowerBarColor[powerType] or PowerBarColor['MANA']
				r, g, b = info.r, info.g, info.b
			else
				r, g, b = altR, altG, altB
			end
		end
	end
	if dead then
		r, g, b = r-.75, g-.75, b-.75
	end
	
	self:SetColor(r, g, b, 1)
end

component.options = {
                    {
                        name = 'Texture',
                        kind = 'Media',
                        key = 'textureName',
                        mediaType = 'StatusBar',
                        handler = string.lower(title),
						panel = "texture",
                    },
}




function component:CreateMenu(menu)
    return bar.Menu.NewMenu(self, menu, 'AltPower', {

                {
                    name = 'Enable',
                    kind = 'CheckButton',
                    key = 'enable',
					panel = "basic",
                },
                {
                    name = 'Opacity',
                    kind = 'Slider',
                    key = 'alpha',
                    min = 0,
                    max = 100,
					panel = "basic",
                },

                {
                    name = 'Texture',
                    kind = 'Media',
                    key = 'textureName',
                    mediaType = 'StatusBar',
                    handler = 'altpower',
					panel = "basic",
                },
                {
                    name = 'Rotate 90',
                    kind = 'CheckButton',
                    key = 'rotate',
					panel = "basic",
                },
                {
                    name = 'Growth',
                    kind = 'Menu',
                    key = 'orient',
                    table = {"HORIZONTAL", "VERTICAL"},
					panel = "basic",
                },
                {
                    name = 'Backdrop',
                    kind = 'CheckButton',
                    key = 'showBG',
					panel = "basic",
                },
                {
                    name = 'Background Alpha',
                    kind = 'Slider',
                    key = 'backdropOpacity',
                    min = 0,
                    max = 100,
					panel = "basic",
                },

                {
                    name = 'Text Anchor',
                    kind = 'Menu',
                    key = 'anchorText',
                    table = {
                        'TopLeft',
                        'Top',
                        'TopRight',
                        'Right',
                        'BottomRight',
                        'Bottom',
                        'BottomLeft',
                        'Left',
                        'Center',
                    },
					panel = "text",
                },
                {
                    name = 'Max Width',
                    kind = 'Slider',
                    key = 'maxTextWidth',
                    min = 1,
                    max = 300,
					panel = "text",
                },
                {
                    name = 'Text Y offset',
                    kind = 'Slider',
                    key = 'textY',
                    min = -150,
                    max = 150,
					panel = "text",
                },
                {
                    name = 'Text X offset',
                    kind = 'Slider',
                    key = 'textX',
                    min = -150,
                    max = 150,
					panel = "text",
                },
                {
                    name = 'Curve offset',
                    kind = 'Slider',
                    key = 'CurveOffset',
                    min = -150,
                    max = 150,
					panel = "text",
                },	
                {
                    name = 'Arc',
                    kind = 'Slider',
                    key = 'arc',
                    min = -300,
                    max = 300,
					panel = "text",
                },
                {
                    name = 'Reverse Arc',
                    kind = 'CheckButton',
                    key = 'reverseArc',
					panel = "text",
                },


                    {
                        name = 'Anchor',
                        kind = 'Menu',
                        key = 'anchor',
                        table = {
                            'TopLeft',
                            'Top',
                            'TopRight',
                            'Right',
                            'BottomRight',
                            'Bottom',
                            'BottomLeft',
                            'Left',
                            'Center',
                        },
						panel = "advanced",
                    },
                    {
                        name = 'Frame Level',
                        kind = 'Slider',
                        key = 'frameLevel',
                        min = 1,
                        max = 100,
						panel = "advanced",
                    },	
                    {
                        name = 'Frame Layer',
                        kind = 'Slider',
                        key = 'frameLayer',
                        min = 1,
                        max = 8,
						panel = "advanced",
                    },
                    {
                        name = 'Width',
                        kind = 'Slider',
                        key = 'width',
                        min = 10,
                        max = 400,
						panel = "advanced",
                    }, 
                    {
                        name = 'Height',
                        kind = 'Slider',
                        key = 'height',
                        min = 10,
                        max = 200,
						panel = "advanced",
                    },
                    {
                        name = 'X Offset',
                        kind = 'Slider',
                        key = 'x',
                        min = -400,
                        max = 400,
						panel = "advanced",
                    },
                    {
                        name = 'Y Offset',
                        kind = 'Slider',
                        key = 'y',
                        min = -400,
                        max = 400,
						panel = "advanced",
                    },
                    {
                        name = 'Tooltip',
                        kind = 'CheckButton',
                        key = 'mouseOverTooltip',
						panel = "advanced",
                    },

        }, 1)
end


