local bar = _G[...]
local modName = ...
local Addon = LibStub('AceAddon-3.0'):GetAddon(GetAddOnDependencies(modName))

--Casting Bar
local component = bar:NewComponent('cast', 'StatusBar')
local arcStrings = bar.ArcStrings

function component:New(parent)
	local name = ('%s_%s_CastingBar'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local opac = CreateFrame('Frame', nil, parent.box)--opacity control
	local bar = self:Bind(CreateFrame('StatusBar', name, opac, 'DU_CastingBarFrameTemplate'))
	bar.owner = parent
	bar.opac = opac
	bar:Hide()
	bar:SetAttribute('unit', parent.id)
	CastingBarFrame_SetLook(bar, 'UNITFRAME')
	bar.Text:SetHeight(12)
	bar.id = parent.id
	bar.time = 	bar.time or 	bar:CreateFontString(nil, 'OVERLAY', 'TextStatusBarText')
	bar.time:SetTextColor(1.0,1.0,1.0)
	bar.time:SetAllPoints(	bar)
	bar.drop = 	bar.drop or CreateFrame('StatusBar', nil, bar)
	bar.drop:SetMinMaxValues(0,1)
	bar.drop:SetValue(1)
	bar.drop:SetAllPoints(true)
	bar.drop:SetFrameLevel(bar:GetFrameLevel()-2)
	bar.spark = bar.Spark
	bar.Spark = bar:CreateTexture(nil, 'ARTWORK')
	bar.Spark:SetPoint("Center")
	bar.Spark :SetSize(1,1)
	bar.Border:SetParent(MainMenuBarArtFrame)
	bar.BorderShield:SetParent(MainMenuBarArtFrame)
	bar.Icon:SetParent(MainMenuBarArtFrame)
	bar.Flash:SetParent(MainMenuBarArtFrame)
	return bar
end

component.defaults = {
	texturePath = 'Interface\\TargetingFrame\\UI-StatusBar',
	textureName = 'Blizzard',
	frameLevel = 2,
	frameLayer = 4,
	scale = 100,
	alpha = 100,
	width = 117,
	y = 17,
	x = 7,
	anchor = 'Left',
	height = 16,
	enable = true,
	showBG = false,
	mouseOverTooltip = true,
	arc = 0,
    textX = 0,
    textY = 0,
	yTextOffset = 0,
	xTextOffset = 0,
	anchorText = "Center",
	maxTextWidth = 98,
	CurveOffset = 0, 
	sparkY = 0,
	Sparkarc = 0,
	multi = true,
    rotate = nil,
	orient = "HORIZONTAL",
	backdropOpacity = 50
}

local layers = {
	'BACKGROUND',
	'LOW',
	'MEDIUM',
	'HIGH',
	'DIALOG',
	'FULLSCREEN',
	'FULLSCREEN_DIALOG',
	'TOOLTIP',
}

function component:Load()
	CastingBarFrame_OnLoad (self, self.id)
	self:SetScript('OnShow', self.OnShow)
	self:SetScript('OnEvent', nil)
	self:Show()
	arcStrings:EnableArcStrings(self)
end

function component:Layout()
	local sets = self.sets
	if sets.enable ~= true then
		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	self:ClearAllPoints()
	self:SetPoint(sets.anchor, self:GetParent():GetParent(), sets.x, sets.y)
	local width, height = (sets.width), (sets.height)
	self:SetSize(width, height)
	if sets.scale < 25 then
		sets.scale = sets.scale *100
	end
	self:SetScale(sets.scale/100)
	self.opac:SetAlpha(sets.alpha/100)
	local anchor = sets.anchor
	local hori = 'CENTER' -- 'LEFT','RIGHT', or 'CENTER'
	local vert = 'MIDDLE' -- 'TOP','BOTTOM', or 'MIDDLE'
	local H, V = 'RIGHT', 'MIDDLE'
--	self.Text:ClearAllPoints()
--	self.Text:SetPoint(anchor)	
	if string.find(anchor, 'Left') then
		hori = 'LEFT'
		H = 'RIGHT'
	elseif string.find(anchor, 'Right') then
		hori = 'RIGHT'
		H = 'LEFT'
	end
	if string.find(anchor, 'Top') then
		vert = 'TOP'
		V = 'TOP'
	elseif string.find(anchor, 'Bottom') then
		vert = 'BOTTOM'
		V = 'BOTTOM'
	end
	if V == 'MIDDLE' then
		V = 'CENTER'
	end
	self.Text:SetJustifyH(hori) -- Sets horizontal text justification ('LEFT','RIGHT', or 'CENTER')
	--self.Text:SetWidth(self:GetWidth()-24)
	self.time:SetJustifyH(H)
	self.time:SetJustifyV(V)
	local lay = layers[sets.frameLayer]
	self:SetFrameStrata(lay)
	self:SetFrameLevel(sets.frameLevel)
    if sets.showBG then
        self.drop:SetStatusBarTexture(self:GetMediaPath('statusbar', self.sets.textureName) or self.sets.texturePath)
        self.drop:SetStatusBarColor(0,0,0,self.sets.backdropOpacity/100)
    else
        self.drop:SetStatusBarTexture("")
    end
	self:SetOrientation(sets.orient) -- "HORIZONTAL" or "VERTICAL"
	self.drop:SetOrientation(sets.orient)
    if (self.prevTexture ~= self:GetMediaPath('statusbar', self.sets.textureName)) or
    (self.prevTexture ~= self.sets.texturePath) then
	    self:SetStatusBarTexture(self:GetMediaPath('statusbar', self.sets.textureName) or self.sets.texturePath)
    end
    self:SetRotatesTexture(self.sets.rotate)
    self.drop:SetRotatesTexture(self.sets.rotate)
    if sets.rotate == true then
        self.spark:SetRotation(math.rad(-90))
    else
        self.spark:SetRotation(math.rad(0))
    end
end

function component:GetMediaPath(kind, fileName)
	if bar.lib then
		self.sets.texturePath = bar.lib and bar.lib:Fetch(kind, fileName)
	end
	return (bar.lib and bar.lib:Fetch(kind, fileName))
end

local function SettingsChangeDetected(self, sets)
    local changeDetected
    for setting, value in pairs(sets) do
        if self["previous-"..setting] ~= value then
            self["previous-"..setting] = value
            changeDetected = true
        end
    end
    if changeDetected then
        return true
    end    
end

local function nochange(self, kind, name, subText, text, texture, startTime, endTime, castID)
    if self.prevkind ~= kind or 
    self.prevname ~= name or
    self.prevsubText ~= subText or
    self.prevtext ~= text or
    self.prevtexture ~= texture or
    self.prevstartTime ~= startTime or
    self.prevendTime ~= endTime or
    self.prevcastID ~= castID
    or SettingsChangeDetected(self, self.sets) then
        self.prevkind = kind
        self.prevname = name
        self.prevsubText = subText
        self.prevtext = text
        self.prevtexture = texture
        self.prevstartTime = startTime
        self.prevendTime = endTime
        self.prevcastID = castID
    else
        return
    end
end

function component:Update(elapsed)
	if self.noUpdate then
		return
	end
	local curGUID = UnitGUID(self.id)
	if (curGUID ~= self.lastGUID) or not curGUID then
		if self.lastGUID then
			self:GetScript('OnShow')(self)
		end
		if not curGUID then
			return
		end
	end
	local kind, name, subText, text, texture, startTime, endTime, _,  castID = self:isCasting()
    
    if nochange(self, kind, name, subText, text, texture, startTime, endTime, castID) then
        return
    end
	self.seconds = self.seconds or ''
	if kind then
		--Events to mimmick
		--cast start
		--cast delayed
		--channel Start
		if kind == 'cast' then
			if not self.startTime then 
				CastingBarFrame_OnEvent (self, 'UNIT_SPELLCAST_START', self.id, name, nil, castID, select( 7, GetSpellInfo(name)))
			else
				if self.endTime ~= endTime then
					CastingBarFrame_OnEvent (self, 'UNIT_SPELLCAST_DELAYED', self.id, self.spellname, nil, self.castID, self.spellID)
				end
			end
		elseif kind == 'channel' then
			CastingBarFrame_OnEvent (self, 'UNIT_SPELLCAST_CHANNEL_START', self.id, name, 0, select( 7, GetSpellInfo(name)))
		end
		self.begin = self.begin or GetTime()
		self.kind = kind
		self.spellID = select( 7, GetSpellInfo(name))
		self.startTime = startTime
		self.endTime = endTime
		self.spellname = name
		if kind == 'cast' then
			self.castID = castID
		else
			self.castID = castID or 0
		end
		self.stop = nil
	elseif self.kind then
		local time = GetTime()
		if ((time) < ((self.endTime/1000) - .5)) then
			if self.kind == 'cast' then
				CastingBarFrame_OnEvent (self, 'UNIT_SPELLCAST_INTERRUPTED', self.id, self.spellname, nil, self.castID, self.spellID)
				self.casting = false
                self.spark:Hide()
			elseif self.kind == 'channel' then
				CastingBarFrame_OnEvent (self, 'UNIT_SPELLCAST_CHANNEL_STOP', self.id, self.spellname, nil, self.castID, self.spellID)
				self.channeling = false
                self.spark:Hide()
			end
		else
			if self.kind == 'cast' then
				CastingBarFrame_OnEvent (self, 'UNIT_SPELLCAST_STOP', self.id, self.spellname, nil, self.castID, self.spellID)
				self.casting = false
                self.spark:Hide()
			elseif self.kind == 'channel' then
				CastingBarFrame_OnEvent (self, 'UNIT_SPELLCAST_CHANNEL_STOP', self.id, self.spellname, nil, self.castID, self.spellID)
				self.channeling = false
                self.spark:Hide()
			end
		end
		self.kind = nil
		self.spellID = nil
		self.startTime = nil
		self.endTime = nil
		self.spellname = nil
		self.begin = nil
		self.seconds = ''
		self.stop = true
	end
	if not name then
        if self.stop then
            self.time:SetText('')
            arcStrings:ClearString(self)
        else
            if self.endTime then
                self.seconds = string.format('%.1f' ,(self.endTime / 1000) - GetTime())
                self.time:SetText(self.seconds)
            end
        end
    end
	if name then
        self.Text:SetTextColor(1, 0, 0, 0)
        self.Text:Hide()
        self.Text:ClearAllPoints()
        self.Text:SetPoint("Center")
        arcStrings:SetText(self, name)
        self.spark:Show()
	end
    if self.value then
        local Sx, Sy = self:GetCenter()
        local width, height = self:GetSize()
        if self.rotated ~= self.sets.rotate then
            self.spark:ClearAllPoints()
        end
        local sparkPosition
        local y
        if ( self.spark ) then
            if self.sets.orient == "HORIZONTAL" then
                sparkPosition = (self.value / self.maxValue) * width;
                y = ((((height/2)/((width/2)^2) )*(((Sx - (Sx + (sparkPosition - (width/2)))))^2)) * (self.sets.Sparkarc/100)) + self.sets.sparkY
			    self.spark:SetPoint("CENTER", self, "LEFT", sparkPosition, y or self.spark.offsetY or 2);
            else
                sparkPosition = (self.value / self.maxValue) * height;
                y = ((((width/2)/((height/2)^2) )*(((Sy - (Sy + (sparkPosition - (height/2)))))^2)) * (self.sets.Sparkarc/100)) + self.sets.sparkY
			    self.spark:SetPoint("CENTER", self, "BOTTOM", y or self.spark.offsetY or 2, sparkPosition );
            end
            self.spark.offsetY = y
		end
    end
	self.lastGUID = curGUID
end

function component:isCasting()
	if UnitCastingInfo(self.id) then
		return 'cast', UnitCastingInfo(self.id)
	elseif UnitChannelInfo(self.id) then
		return 'channel', UnitChannelInfo(self.id)
	end
end

function component:OnShow()
	local kind,_, _, _, _, startTime, endTime = self:isCasting()
	if (kind == 'cast') then
		if ( startTime ) then
			self.value = (GetTime() - (startTime / 1000))
		end
	elseif (kind == 'channel') then
		if ( endTime ) then
			self.value = ((endTime / 1000) - GetTime())
		end
	else
		self.fadeOut = 0
		self.holdTime = 0
		self:Hide()
	end
end

function component:UpdateValue(key, value)
	self.sets[key] = value
	self:Layout()
end

function component:GetValue(key)
	return self.sets[key]
end

local anchors = {
	'TopLeft',
	'Top',
	'TopRight',
	'Right',
	'BottomRight',
	'Bottom',
	'BottomLeft',
	'Left',
	'Center',
}

function component:CreateMenu(menu)
    return bar.Menu.NewMenu(self, menu, 'Cast', {

                {
                    name = 'Enable',
                    kind = 'CheckButton',
                    key = 'enable',
					panel = "basic",
                },
                {
                    name = 'Opacity',
                    kind = 'Slider',
                    key = 'alpha',
                    min = 0,
                    max = 100,
					panel = "basic",
                },

                {
                    name = 'Texture',
                    kind = 'Media',
                    key = 'textureName',
                    mediaType = 'StatusBar',
                    handler = 'cast',
					panel = "basic",
                },
                {
                    name = 'Rotate 90',
                    kind = 'CheckButton',
                    key = 'rotate',
					panel = "basic",
                },
                {
                    name = 'Growth',
                    kind = 'Menu',
                    key = 'orient',
                    table = {"HORIZONTAL", "VERTICAL"},
					panel = "basic",
                },
                {
                    name = 'Backdrop',
                    kind = 'CheckButton',
                    key = 'showBG',
					panel = "basic",
                },
                {
                    name = 'Background Alpha',
                    kind = 'Slider',
                    key = 'backdropOpacity',
                    min = 0,
                    max = 100,
					panel = "basic",
                },

                {
                    name = 'Text Anchor',
                    kind = 'Menu',
                    key = 'anchorText',
                    table = {
                        'TopLeft',
                        'Top',
                        'TopRight',
                        'Right',
                        'BottomRight',
                        'Bottom',
                        'BottomLeft',
                        'Left',
                        'Center',
                    },
					panel = "text",
                },
                {
                    name = 'Max Width',
                    kind = 'Slider',
                    key = 'maxTextWidth',
                    min = 1,
                    max = 300,
					panel = "text",
                },
                {
                    name = 'Text Y offset',
                    kind = 'Slider',
                    key = 'textY',
                    min = -150,
                    max = 150,
					panel = "text",
                },
                {
                    name = 'Text X offset',
                    kind = 'Slider',
                    key = 'textX',
                    min = -150,
                    max = 150,
					panel = "text",
                },
                {
                    name = 'Curve offset',
                    kind = 'Slider',
                    key = 'CurveOffset',
                    min = -150,
                    max = 150,
					panel = "text",
                },	
                {
                    name = 'Arc',
                    kind = 'Slider',
                    key = 'arc',
                    min = -300,
                    max = 300,
					panel = "text",
                },
                {
                    name = 'Reverse Arc',
                    kind = 'CheckButton',
                    key = 'reverseArc',
					panel = "text",
                },

                {
                    name = 'Spark Y offset',
                    kind = 'Slider',
                    key = 'sparkY',
                    min = -150,
                    max = 150,
					panel = "spark",
                },
                {
                    name = 'Spark Arc',
                    kind = 'Slider',
                    key = 'Sparkarc',
                    min = -300,
                    max = 300,
					panel = "spark",
                },

                    {
                        name = 'Anchor',
                        kind = 'Menu',
                        key = 'anchor',
                        table = {
                            'TopLeft',
                            'Top',
                            'TopRight',
                            'Right',
                            'BottomRight',
                            'Bottom',
                            'BottomLeft',
                            'Left',
                            'Center',
                        },
						panel = "advanced",
                    },
                    {
                        name = 'Frame Level',
                        kind = 'Slider',
                        key = 'frameLevel',
                        min = 1,
                        max = 100,
						panel = "advanced",
                    },	
                    {
                        name = 'Frame Layer',
                        kind = 'Slider',
                        key = 'frameLayer',
                        min = 1,
                        max = 8,
						panel = "advanced",
                    },
                    {
                        name = 'Width',
                        kind = 'Slider',
                        key = 'width',
                        min = 10,
                        max = 400,
						panel = "advanced",
                    }, 
                    {
                        name = 'Height',
                        kind = 'Slider',
                        key = 'height',
                        min = 10,
                        max = 200,
						panel = "advanced",
                    },
                    {
                        name = 'X Offset',
                        kind = 'Slider',
                        key = 'x',
                        min = -400,
                        max = 400,
						panel = "advanced",
                    },
                    {
                        name = 'Y Offset',
                        kind = 'Slider',
                        key = 'y',
                        min = -400,
                        max = 400,
						panel = "advanced",
                    },
                    {
                        name = 'Tooltip',
                        kind = 'CheckButton',
                        key = 'mouseOverTooltip',
						panel = "advanced",
                    },

        }, 1)
end
