local bar = _G[...]
local modName = ...

--generic StatusBar
local template = bar:NewTemplate("StatusBar")

local arcStrings = bar.ArcStrings

function template:CreateNew(parent, title)
	local name = ('%s_%s_'..title):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local statusBar = CreateFrame('StatusBar', name, parent.box, 'TextStatusBar')
	statusBar.owner = parent
	statusBar:SetStatusBarTexture('Interface\\RaidFrame\\Raid-Bar-Hp-Fill', 'BORDER')
	statusBar:SetStatusBarColor(0,1,0,1)
	statusBar:EnableMouse(false)
	statusBar:Show()
	statusBar.title = title
	statusBar:SetScript('OnEnter', nil)
	statusBar.text = statusBar.text or statusBar:CreateFontString(nil, 'ARTWORK', 'TextStatusBarText')
	statusBar.text:SetTextColor(1.0,1.0,1.0)
	statusBar.text:SetAllPoints(true)
	statusBar.drop = 	statusBar.drop or CreateFrame('StatusBar', nil, statusBar)
	statusBar.drop:SetMinMaxValues(0,1)
	statusBar.drop:SetValue(1)
	statusBar.drop:SetAllPoints(true)
	statusBar.drop:SetFrameLevel(statusBar:GetFrameLevel()-2)
	return statusBar
end

function template:Load()
    arcStrings:EnableArcStrings(self)
end

template.defaults = {
	scale = 100,
	height = 12,
	justifyV = 'CENTER',
	width = 51,
	x = -106,
	y = -41,
	justifyH = 'CENTER',
	drop = true,
	anchor = 'TopRight',
	enable = true,
	percent = true,
	alpha = 100,
	format = 'value',
	mouseover = 'value',
	textureName = 'Raid',
	texturePath = 'Interface\\RaidFrame\\Raid-Bar-Hp-Fill',
	textHori = 2,
	textVert = 2,
	orient = "HORIZONTAL",
	prevOrient = "HORIZONTAL",
	anchorText = "Center",
    mouseOverTooltip = true,
	arc = 0,
	textX = 0,
	textY = 0,
	maxTextWidth = 98,
	reverseArc = nil,
	maxTextWidth = 1000,
    CurveOffset = 0,
    topOffset = 0,
    bottomOffset = 100,
    leftOffset = 0,
    rightOffset = 100,
    backdropOpacity = 50,
	LifeTime = 0,
	CurrentPlayTime = 0,
}
	
local hori = {'LEFT', 'CENTER', 'RIGHT'}
local vert = {'TOP',  'MIDDLE', 'BOTTOM'}

function template:Layout()
	local sets = self.sets
	if sets.enable ~= true then
		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	local sets = self.sets
	self:Reposition()
	self:Rescale()
    self:SetOrientation(sets.orient) -- "HORIZONTAL" or "VERTICAL"
	self.drop:SetOrientation(sets.orient)
	local width, height = (sets.width), (sets.height)
	if sets.prevOrient ~= sets.orient then
	    sets.prevOrient = sets.orient 
	    width, height = height, width
	end
	self:SetSize(width, height)
	self:SetAlpha(sets.alpha/100)
	self.text:ClearAllPoints()
    local w, h = self:GetSize()
    local X, Y = w*(sets.textX/100), h*(sets.textY/100)
	self.text:SetPoint(sets.anchorText, X, Y)
--	if (self.sets.textureName ~= self.prevtextureName) or (self.sets.texturePath ~=self.prevtexturePath) then
        self.prevtextureName = self.sets.textureName
        self.prevtexturePath = self.sets.texturePath
        if sets.drop then
            self.drop:Show()
        else
            self.drop:Hide()
        end
        self:SetStatusBarTexture(self:GetMediaPath('statusbar', self.sets.textureName) or self.sets.texturePath)
        self.drop:SetStatusBarTexture( self:GetMediaPath('statusbar', self.sets.textureName) or self.sets.texturePath)
        self.drop:SetStatusBarColor(0,0,0,self.sets.backdropOpacity/100)
        self:SetRotatesTexture(self.sets.rotate)
        self.drop:SetRotatesTexture(self.sets.rotate)
--      left, right, top, bottom = self.sets.topOffset/100, self.sets.bottomOffset/100, self.sets.leftOffset/100, self.sets.rightOffset/100
--      self:GetStatusBarTexture():SetTexCoord(left, right, top, bottom)
--  end
	self:Update()
end

local function FormatValue(value)
	if (value <= 1000) then
		return value
	elseif (value < 1000000) then
		return ('%.1fk'):format(value / 1000);
	elseif (value < 1000000000) then
		return ('%.2fm'):format(value / 1000000);
	else
		return ('%.2fg'):format(value / 1000000000);
	end
end

local function FormatBarValues(value,max,type)
	if (type == 'none') then
		return ''
	elseif (type == 'value') or (max == 0) then
		return string.format('%s / %s',AbbreviateLargeNumbers(value),AbbreviateLargeNumbers(max))
	elseif (type == 'current') then
		return string.format('%s',AbbreviateLargeNumbers(value))
	elseif (type == 'full') then
		return string.format('%s / %s (%.0f%%)',FormatValue(value),FormatValue(max),value / max * 100)
	elseif (type == 'deficit') then
		if (value ~= max) then
			return string.format('-%s',FormatValue(max - value))
		else
			return ''
		end
	elseif (type == 'percent') then
		return string.format('%.0f%%',value / max * 100)
	end
end

local function SettingsChangeDetected(self, sets)
    local changeDetected
    for setting, value in pairs(sets) do
        if self["previous-"..setting] ~= value then
            self["previous-"..setting] = value
            changeDetected = true
        end
    end
    if changeDetected then
        return true
    end    
end

local function nochange(self, unit, min, max, current, dead, sets)
    if 	self.prevunit ~= unit
	or self.prevmax ~= max
	or self.prevmin ~= min
	or self.prevcurrent ~= current
	or self.prevdead ~= dead
    or (self.prevarc ~= self.sets.arc)
    or (arcStrings:RelocateStringManager(self))
    or SettingsChangeDetected(self, sets)
    then  
        self.prevunit = unit
        self.prevmax = max
        self.prevmin = min
        self.prevcurrent = current
        self.prevdead = dead
	else
	    return true
	end
end

function template:Update()
	if self.noUpdate then
		return
	end
	if self.OnUpdate then
		self:OnUpdate()
	end
end

function template:SetValues(current, minimum, maximum, overide, overideText)
	self:SetMinMaxValues(minimum, maximum)
	if overideText and self.SetText then
		self:SetValue(max(minimum, 0))
		self:SetText(overideText)
		self.text:Hide()
		arcStrings:SetText(self, overideText)
		return	
	end
	local sets = self.sets
	if self.sets.mouseover ~= 'none' and (MouseIsOver(self)) then
		self.text:SetText(FormatBarValues(current, maximum, self.sets.mouseover))
        arcStrings:SetText(self, tostring(FormatBarValues(current, maximum, self.sets.mouseover)))
	else
		self.text:SetText(FormatBarValues(current, maximum, self.sets.format))
        arcStrings:SetText(self, tostring(FormatBarValues(current, maximum, self.sets.format)))
	end
	self.text:Hide()
	self:SetValue(current)
end

function template:SetColor(r, g, b, a)
	if ( r ~= self.r or g ~= self.g or b ~= self.b) or (a ~= self.a) then
		self:SetStatusBarColor(r, g, b)
		self.r, self.g, self.b, self.a = r, g, b, a
	end
end

function template:Rescale()
	self:SetScale(self.sets.scale/100)
end

function template:Reposition()
	self:ClearAllPoints()
	local x = self.sets.x/(self.sets.scale/100)
	local y = self.sets.y/(self.sets.scale/100)
	self:SetPoint(self.sets.anchor, self:GetParent(), x, y)
end

function template:GetMediaPath(kind, fileName)
	if bar.lib then
		self.sets.texturePath = bar.lib and bar.lib:Fetch(kind, fileName)
	end
	return (bar.lib and bar.lib:Fetch(kind, fileName))
end

local function Merge(source, destination)
	if not destination then
		destination = {}
	end
	if not source then
		return destination
	end
	for i, b in pairs(CopyTable(source)) do
		if (type(b) == 'table') then
			destination[i] = Merge(b, destination[i])
		else
			destination[i] = b
		end
	end
	return destination
end

function template:CreateMenu(menu)
	local options = {

   --[[                 {
                        name = 'Class Colored',
                        kind = 'CheckButton',
                        key = 'classColor',
						panel = "texture",
                    },
--]]
                    {
                        name = 'Backdrop',
                        kind = 'CheckButton',
                        key = 'drop',
						panel = "texture",
                    },
                    {
                        name = 'Rotate 90',
                        kind = 'CheckButton',
                        key = 'rotate',
						panel = "texture",
                    },
                    {
                        name = 'Growth',
                        kind = 'Menu',
                        key = 'orient',
                        table = {"HORIZONTAL", "VERTICAL"},
						panel = "texture",
                    },
 --[[                   {
                        name = 'Texture',
                        kind = 'Media',
                        key = 'textureName',
                        mediaType = 'StatusBar',
                        handler = 'health',
						panel = "texture",
                    },
--]]
                    {
                        name = 'Background Alpha',
                        kind = 'Slider',
                        key = 'backdropOpacity',
                        min = 0,
                        max = 100,
						panel = "texture",
                    },	

                    {
                        name = 'Format',
                        kind = 'Menu',
                        key = 'format',
                        table = {
                            'none',
                            'value',
                            'current',
                            'full',
                            'deficit',
                            'percent',
                        },
						panel = "text",
                    },
                    {
                        name = 'MouseOver Format',
                        kind = 'Menu',
                        key = 'mouseover',
                        table = {
                            'none',
                            'value',
                            'current',
                            'full',
                            'deficit',
                            'percent',
                        },
						panel = "text",
                    },
                    {
                        name = 'Text Anchor',
                        kind = 'Menu',
                        key = 'anchorText',
                        table = {
                            'TopLeft',
                            'Top',
                            'TopRight',
                            'Right',
                            'BottomRight',
                            'Bottom',
                            'BottomLeft',
                            'Left',
                            'Center',
                        },
						panel = "text",
                    },
                    {
                        name = 'Max Width',
                        kind = 'Slider',
                        key = 'maxTextWidth',
                        min = 24,
                        max = 300,
						panel = "text",
                    },
                    {
                        name = 'Text Y offset',
                        kind = 'Slider',
                        key = 'textY',
                        min = -150,
                        max = 150,
						panel = "text",
                    },	
                    {
                        name = 'Text X offset',
                        kind = 'Slider',
                        key = 'textX',
                        min = -150,
                        max = 150,
						panel = "text",
                    },
                    {
                        name = 'Curve offset',
                        kind = 'Slider',
                        key = 'CurveOffset',
                        min = -400,
                        max = 400,
						panel = "text",
                    },
                    {
                        name = 'Arc',
                        kind = 'Slider',
                        key = 'arc',
                        min = -100,
                        max = 100,
						panel = "text",
                    },
                    {
                        name = 'Reverse Arc',
                        kind = 'CheckButton',
                        key = 'reverseArc',
						panel = "text",
                    },
                    {
                        name = 'Enable',
                        kind = 'CheckButton',
                        key = 'enable',
						panel = "basic",
                    },
                    {
                        name = 'Opacity',
                        kind = 'Slider',
                        key = 'alpha',
                        min = 0,
                        max = 100,
						panel = "basic",
                    },
                    {
                        name = 'Width',
                        kind = 'Slider',
                        key = 'width',
                        min = 5,
                        max = 250,
						panel = "basic",
                    },
                    {
                        name = 'Height',
                        kind = 'Slider',
                        key = 'height',
                        min = 5,
                        max = 250,
						panel = "basic",
                    },
                    {
                        name = 'Scale',
                        kind = 'Slider',
                        key = 'scale',
                        min = 25,
                        max = 250,
						panel = "basic",
                    },

                    {
                        name = 'Tooltip',
                        kind = 'CheckButton',
                        key = 'mouseOverTooltip',
						panel = "advanced",
                    },
                    {
                        name = 'Anchor',
                        kind = 'Menu',
                        key = 'anchor',
                        table = {
                            'TopLeft',
                            'Top',
                            'TopRight',
                            'Right',
                            'BottomRight',
                            'Bottom',
                            'BottomLeft',
                            'Left',
                            'Center',
                        },
						panel = "advanced",
                    },


                    {
                        name = 'X Offset',
                        kind = 'Slider',
                        key = 'x',
                        min = -400,
                        max = 400,
						panel = "advanced",
                    },
                    {
                        name = 'Y Offset',
                        kind = 'Slider',
                        key = 'y',
                        min = -400,
                        max = 400,
						panel = "advanced",
                    },
        }

	if self.options then
		Merge(options, self.options)
	end		

	if self.AddMenu then
		self:AddMenu()
	else
		return bar.Menu.NewMenu(self, menu, self.title, options, 1)
	end
end
