local bar = _G[...]
local mod = bar.master:NewModule('Frame')

local enabledFrames = {}

local defaults  = {
	frames = {
		player = true,
		target = true,
		targettarget = true,
		focus = true,
		focustarget = true,
		playerpet = true,
	},
	hide = {
		PlayerFrame = true,
		TargetFrame = true,
		FocusFrame = true,
	}
}

local function getSets()
	bar.master.db.profile.units = bar.master.db.profile.units or defaults
	bar.master.db.profile.units.hide = bar.master.db.profile.units.hide or bar.master.db.profile.units.disable or {
		PlayerFrame = true,
		TargetFrame = true,
		FocusFrame = true,
	}
	return bar.master.db.profile.units
end

bar.getSets = getSets

local def = {
			scale = 1,
			showInOverrideUI = false,
			showInPetBattleUI = false,
			point = 'Center',
			padW = 3,
			width = 232,
			y = 0,
			x = 0,
			padH = 3,
			height = 100,
			focus = {
				x = -42,
				y = -12,
				anchor = 'TopRight',
			},
			power = {
				anchor = 'TopRight',
				x = -106,
				y = -52,
				height = 12,
				width = (119/232)*100,
			},
			health = {
				anchor = 'TopRight',
				x = -106,
				y = -41,
				height = 12,
				width = (119/232)*100,
			},
			focus = {
				x = -42,
				y = -12,
				anchor = 'TopRight',
			},
			name = {
				width = 100,
				height = 12,
				anchor = 'Center',
				x = -50,
				y = 19,
				justifyH = 2,
			},
			icons = {
				width = 45,
				height = 45,
				style = 'ROUND',
				scale = 130,
				anchor = 'Right',
				angle  = 313,
				x = -45,
				y = 2,
				IsTopToBottom = true,
				IsLeftToRight = false,
			},
			level = {
				anchor = 'Center',
				x = 63,
				y = -16,
				justifyH = 2,				
			},
			background = {
				anchor = 'TopRight',
				x = -106,
				y = -22,
				width = (119/232)*100,
				height = 41,
				color = {
						r = 0,
						g = 0,
						b = 0,
						a = .5, 
				}
			},
			
			buffs = {
				enable = true,
				anchor = 'BottomLeft',
				x = 12,
				y = 6,
				scale = 64,
				rows = 1,
			},
			debuffs = {
				enable = true,
			},
			cast = {	
				frameLevel = 2,
				frameLayer = 4,
				scale = 100,
				alpha = 100,
				width = 117,
				y = 17,
				x = 7,
				anchor = 'Left',
				height = 16,
				enable = true,
				showBG = false,
				textheight = 12,
			},
		}

function mod:OnInitialize()
	bar.master.db.profile.units = bar.master.db.profile.units or defaults
	bar.unitLayouts = nil
	bar.unitLayouts = {Default = bar:GetDefaults()}
end

function mod:Load()
	wipe(enabledFrames)
	for unit, b in pairs(getSets().frames) do
		unit = string.lower(unit)
		local frame = bar:New(unit)
		bar.unitLayouts[unit] = frame.sets
		tinsert(enabledFrames, frame)
	end	

	for frameName, b in pairs(getSets().hide) do
		if frameName and _G[frameName] then
			_G[frameName]:SetParent(MainMenuBarArtFrame)
		end
	end
end

function mod:Unload()
	if enabledFrames then
		for i, frame in pairs(enabledFrames) do
			bar.master.Frame:ForFrame(frame.id, 'Free')
			
				frame:Unload()
			
		end
	end
end


local lib = LibStub('LibSharedMedia-3.0')
--bar.statusbarTextures = {
--    left = {
--        ["ArcHUD"] = [[Interface\Addons\Dominos_Units\tex\bars\ArcHUD]],
--        ["Bar"] = [[Interface\Addons\Dominos_Units\tex\bars\Bar]],
--        ["BloodGlaives"] = [[Interface\Addons\Dominos_Units\tex\bars\BloodGlaives]],
--        ["CleanCurvesOut"] = [[Interface\Addons\Dominos_Units\tex\bars\CleanCurvesOut]],
--        ["ColorBar"] = [[Interface\Addons\Dominos_Units\tex\bars\ColorBar]],
--        ["FangRune"] = [[Interface\Addons\Dominos_Units\tex\bars\FangRune]],
--        ["GlowArc"] = [[Interface\Addons\Dominos_Units\tex\bars\GlowArc]],
--        ["HiBar"] = [[Interface\Addons\Dominos_Units\tex\bars\HiBar]],
--        ["RoundBar"] = [[Interface\Addons\Dominos_Units\tex\bars\RoundBar]]
--    },
--    right = {
-- --       ["ArcHUD-Right"] = [[Interface\Addons\Dominos_Units\tex\bars\ArcHUD-Right]],
--        ["Bar-Right"] = [[Interface\Addons\Dominos_Units\tex\bars\Bar-Right]],
--        ["BloodGlaives-Right"] = [[Interface\Addons\Dominos_Units\tex\bars\BloodGlaives-Right]],
 --       ["CleanCurvesOut-Right"] = [[Interface\Addons\Dominos_Units\tex\bars\CleanCurvesOut-Right]],
--        ["FangRune-Right"] = [[Interface\Addons\Dominos_Units\tex\bars\FangRune-Right]],
--        ["GlowArc-Right"] = [[Interface\Addons\Dominos_Units\tex\bars\GlowArc-Right]],
--        ["HiBar-Right"] = [[Interface\Addons\Dominos_Units\tex\bars\HiBar-Right]],
--        ["RoundBar-Right"] = [[Interface\Addons\Dominos_Units\tex\bars\RoundBar-Right]]
--    },
--}

bar.statusbarTextures = {
    ArcHUD = [[Interface\AddOns\Dominos_Units\textures\ArcHUD.tga]],
    ArcHUD2 = [[Interface\AddOns\Dominos_Units\textures\ArcHUD2.tga]],
    Bar = [[Interface\AddOns\Dominos_Units\textures\Bar.tga]],
    Bar2 = [[Interface\AddOns\Dominos_Units\textures\Bar2.tga]],
    BloodGlaives = [[Interface\AddOns\Dominos_Units\textures\BloodGlaives.tga]],
    BloodGlaives2 = [[Interface\AddOns\Dominos_Units\textures\BloodGlaives2.tga]],
    CleanCurves = [[Interface\AddOns\Dominos_Units\textures\CleanCurves.tga]],
    CleanCurves2 = [[Interface\AddOns\Dominos_Units\textures\CleanCurves2.tga]],
    CleanCurvesOut = [[Interface\AddOns\Dominos_Units\textures\CleanCurvesOut.tga]],
    CleanCurvesOut2 = [[Interface\AddOns\Dominos_Units\textures\CleanCurvesOut2.tga]],
    CleanTank = [[Interface\AddOns\Dominos_Units\textures\CleanTank.tga]],
    CleanTank2 = [[Interface\AddOns\Dominos_Units\textures\CleanTank2.tga]],
    ColorBar = [[Interface\AddOns\Dominos_Units\textures\ColorBar.tga]],
    ColorBar2 = [[Interface\AddOns\Dominos_Units\textures\ColorBar2.tga]],
    ComboCleanCurves = [[Interface\AddOns\Dominos_Units\textures\ComboCleanCurves.blp]],
    DHUD = [[Interface\AddOns\Dominos_Units\textures\DHUD.tga]],
    DHUD2 = [[Interface\AddOns\Dominos_Units\textures\DHUD2.tga]],
    FangRune = [[Interface\AddOns\Dominos_Units\textures\FangRune.tga]],
    FangRune2 = [[Interface\AddOns\Dominos_Units\textures\FangRune2.tga]],
    GemTank = [[Interface\AddOns\Dominos_Units\textures\GemTank.tga]],
    GlowArc = [[Interface\AddOns\Dominos_Units\textures\GlowArc.tga]],
    GlowArc2 = [[Interface\AddOns\Dominos_Units\textures\GlowArc2.tga]],
    HiBar = [[Interface\AddOns\Dominos_Units\textures\HiBar.tga]],
    HiBar2 = [[Interface\AddOns\Dominos_Units\textures\HiBar2.tga]],
    PillTank = [[Interface\AddOns\Dominos_Units\textures\PillTank.tga]],
    RivetBar = [[Interface\AddOns\Dominos_Units\textures\RivetBar.tga]],
    RivetBar2 = [[Interface\AddOns\Dominos_Units\textures\RivetBar2.tga]],
    RoundBar = [[Interface\AddOns\Dominos_Units\textures\RoundBar.tga]],
    RoundBar2 = [[Interface\AddOns\Dominos_Units\textures\RoundBar2.tga]],
    RuneBar = [[Interface\AddOns\Dominos_Units\textures\RuneBar.tga]],
    RuneBar2 = [[Interface\AddOns\Dominos_Units\textures\RuneBar2.tga]],
}



    for textureName, texturePath in pairs(bar.statusbarTextures) do
        if not lib.MediaTable.statusbar[textureName] then
            lib.MediaTable.statusbar[textureName] = texturePath
        end
    end
